/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIHttpChannelChild.idl
 */

#ifndef __gen_nsIHttpChannelChild_h__
#define __gen_nsIHttpChannelChild_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIPrincipal; /* forward declaration */

class nsIURI; /* forward declaration */


/* starting interface:    nsIHttpChannelChild */
#define NS_IHTTPCHANNELCHILD_IID_STR "893e29fb-2e84-454e-afc7-41fadbe93fd9"

#define NS_IHTTPCHANNELCHILD_IID \
  {0x893e29fb, 0x2e84, 0x454e, \
    { 0xaf, 0xc7, 0x41, 0xfa, 0xdb, 0xe9, 0x3f, 0xd9 }}

class NS_NO_VTABLE nsIHttpChannelChild : public nsISupports {
 public:

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IHTTPCHANNELCHILD_IID)

  /* void addCookiesToRequest (); */
  NS_IMETHOD AddCookiesToRequest(void) = 0;

  /* void forceIntercepted (); */
  NS_IMETHOD ForceIntercepted(void) = 0;

  /* readonly attribute RequestHeaderTuples clientSetRequestHeaders; */
  NS_IMETHOD GetClientSetRequestHeaders(mozilla::net::RequestHeaderTuples **aClientSetRequestHeaders) = 0;

  /* [nostdcall,notxpcom] void GetClientSetCorsPreflightParameters (in OptionalCorsPreflightArgsRef args); */
  virtual void GetClientSetCorsPreflightParameters(mozilla::OptionalCorsPreflightArgs & args) = 0;

  /* void removeCorsPreflightCacheEntry (in nsIURI aURI, in nsIPrincipal aRequestingPrincipal); */
  NS_IMETHOD RemoveCorsPreflightCacheEntry(nsIURI *aURI, nsIPrincipal *aRequestingPrincipal) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIHttpChannelChild, NS_IHTTPCHANNELCHILD_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIHTTPCHANNELCHILD \
  NS_IMETHOD AddCookiesToRequest(void) override; \
  NS_IMETHOD ForceIntercepted(void) override; \
  NS_IMETHOD GetClientSetRequestHeaders(mozilla::net::RequestHeaderTuples **aClientSetRequestHeaders) override; \
  virtual void GetClientSetCorsPreflightParameters(mozilla::OptionalCorsPreflightArgs & args) override; \
  NS_IMETHOD RemoveCorsPreflightCacheEntry(nsIURI *aURI, nsIPrincipal *aRequestingPrincipal) override; 

/* Use this macro when declaring the members of this interface when the
   class doesn't implement the interface. This is useful for forwarding. */
#define NS_DECL_NON_VIRTUAL_NSIHTTPCHANNELCHILD \
  NS_METHOD AddCookiesToRequest(void); \
  NS_METHOD ForceIntercepted(void); \
  NS_METHOD GetClientSetRequestHeaders(mozilla::net::RequestHeaderTuples **aClientSetRequestHeaders); \
  void GetClientSetCorsPreflightParameters(mozilla::OptionalCorsPreflightArgs & args); \
  NS_METHOD RemoveCorsPreflightCacheEntry(nsIURI *aURI, nsIPrincipal *aRequestingPrincipal); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIHTTPCHANNELCHILD(_to) \
  NS_IMETHOD AddCookiesToRequest(void) override { return _to AddCookiesToRequest(); } \
  NS_IMETHOD ForceIntercepted(void) override { return _to ForceIntercepted(); } \
  NS_IMETHOD GetClientSetRequestHeaders(mozilla::net::RequestHeaderTuples **aClientSetRequestHeaders) override { return _to GetClientSetRequestHeaders(aClientSetRequestHeaders); } \
  virtual void GetClientSetCorsPreflightParameters(mozilla::OptionalCorsPreflightArgs & args) override { return _to GetClientSetCorsPreflightParameters(args); } \
  NS_IMETHOD RemoveCorsPreflightCacheEntry(nsIURI *aURI, nsIPrincipal *aRequestingPrincipal) override { return _to RemoveCorsPreflightCacheEntry(aURI, aRequestingPrincipal); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIHTTPCHANNELCHILD(_to) \
  NS_IMETHOD AddCookiesToRequest(void) override { return !_to ? NS_ERROR_NULL_POINTER : _to->AddCookiesToRequest(); } \
  NS_IMETHOD ForceIntercepted(void) override { return !_to ? NS_ERROR_NULL_POINTER : _to->ForceIntercepted(); } \
  NS_IMETHOD GetClientSetRequestHeaders(mozilla::net::RequestHeaderTuples **aClientSetRequestHeaders) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetClientSetRequestHeaders(aClientSetRequestHeaders); } \
  virtual void GetClientSetCorsPreflightParameters(mozilla::OptionalCorsPreflightArgs & args) override; \
  NS_IMETHOD RemoveCorsPreflightCacheEntry(nsIURI *aURI, nsIPrincipal *aRequestingPrincipal) override { return !_to ? NS_ERROR_NULL_POINTER : _to->RemoveCorsPreflightCacheEntry(aURI, aRequestingPrincipal); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsHttpChannelChild : public nsIHttpChannelChild
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIHTTPCHANNELCHILD

  nsHttpChannelChild();

private:
  ~nsHttpChannelChild();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsHttpChannelChild, nsIHttpChannelChild)

nsHttpChannelChild::nsHttpChannelChild()
{
  /* member initializers and constructor code */
}

nsHttpChannelChild::~nsHttpChannelChild()
{
  /* destructor code */
}

/* void addCookiesToRequest (); */
NS_IMETHODIMP nsHttpChannelChild::AddCookiesToRequest()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void forceIntercepted (); */
NS_IMETHODIMP nsHttpChannelChild::ForceIntercepted()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute RequestHeaderTuples clientSetRequestHeaders; */
NS_IMETHODIMP nsHttpChannelChild::GetClientSetRequestHeaders(mozilla::net::RequestHeaderTuples **aClientSetRequestHeaders)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [nostdcall,notxpcom] void GetClientSetCorsPreflightParameters (in OptionalCorsPreflightArgsRef args); */
void nsHttpChannelChild::GetClientSetCorsPreflightParameters(mozilla::OptionalCorsPreflightArgs & args)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void removeCorsPreflightCacheEntry (in nsIURI aURI, in nsIPrincipal aRequestingPrincipal); */
NS_IMETHODIMP nsHttpChannelChild::RemoveCorsPreflightCacheEntry(nsIURI *aURI, nsIPrincipal *aRequestingPrincipal)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIHttpChannelChild_h__ */
