/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"
#include "nsITreeView.idl"

interface nsIX509Cert;
interface nsIX509CertList;

[scriptable, uuid(d0180863-606e-49e6-8324-cf45ed4dd891)]
interface nsICertTreeItem : nsISupports {
  readonly attribute nsIX509Cert cert;
  readonly attribute AString hostPort;
};

[scriptable, uuid(55d5ad6b-5572-47fe-941c-f01fe723659e)]
interface nsICertTree : nsITreeView {

  void loadCerts(in unsigned long type);
  void loadCertsFromCache(in nsIX509CertList cache, in unsigned long type);

  nsIX509Cert getCert(in unsigned long index);
  nsICertTreeItem getTreeItem(in unsigned long index);
  boolean isHostPortOverride(in unsigned long index);

  void deleteEntryObject(in unsigned long index);
};

%{C++

#define NS_CERTTREE_CID { 0x4ea60761, 0x31d6, 0x491d, \
                         { 0x9e, 0x34, 0x4b, 0x53, 0xa2, 0x6c, 0x41, 0x6c } }

#define NS_CERTTREE_CONTRACTID "@mozilla.org/security/nsCertTree;1"

%}

