/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

interface nsIDOMDOMRequest;

[scriptable, uuid(44fb7308-7d7b-4975-8a27-e01fe9623bdb)]
interface nsINavigatorPayment : nsISupports
{
  // The 'jwts' parameter can be either a single DOMString or an array of
  // DOMStrings. In both cases, it represents the base64url encoded and
  // digitally signed payment information. Each payment provider should
  // define its supported JWT format.
  nsIDOMDOMRequest pay(in jsval jwts);
};
