/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIAboutNewTabService.idl
 */

#ifndef __gen_nsIAboutNewTabService_h__
#define __gen_nsIAboutNewTabService_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIAboutNewTabService */
#define NS_IABOUTNEWTABSERVICE_IID_STR "6c66f022-beb1-46ea-8af6-c6c6dd937ea9"

#define NS_IABOUTNEWTABSERVICE_IID \
  {0x6c66f022, 0xbeb1, 0x46ea, \
    { 0x8a, 0xf6, 0xc6, 0xc6, 0xdd, 0x93, 0x7e, 0xa9 }}

class NS_NO_VTABLE nsIAboutNewTabService : public nsISupports {
 public:

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IABOUTNEWTABSERVICE_IID)

  /* attribute ACString newTabURL; */
  NS_IMETHOD GetNewTabURL(nsACString & aNewTabURL) = 0;
  NS_IMETHOD SetNewTabURL(const nsACString & aNewTabURL) = 0;

  /* readonly attribute bool overridden; */
  NS_IMETHOD GetOverridden(bool *aOverridden) = 0;

  /* void resetNewTabURL (); */
  NS_IMETHOD ResetNewTabURL(void) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIAboutNewTabService, NS_IABOUTNEWTABSERVICE_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIABOUTNEWTABSERVICE \
  NS_IMETHOD GetNewTabURL(nsACString & aNewTabURL) override; \
  NS_IMETHOD SetNewTabURL(const nsACString & aNewTabURL) override; \
  NS_IMETHOD GetOverridden(bool *aOverridden) override; \
  NS_IMETHOD ResetNewTabURL(void) override; 

/* Use this macro when declaring the members of this interface when the
   class doesn't implement the interface. This is useful for forwarding. */
#define NS_DECL_NON_VIRTUAL_NSIABOUTNEWTABSERVICE \
  NS_METHOD GetNewTabURL(nsACString & aNewTabURL); \
  NS_METHOD SetNewTabURL(const nsACString & aNewTabURL); \
  NS_METHOD GetOverridden(bool *aOverridden); \
  NS_METHOD ResetNewTabURL(void); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIABOUTNEWTABSERVICE(_to) \
  NS_IMETHOD GetNewTabURL(nsACString & aNewTabURL) override { return _to GetNewTabURL(aNewTabURL); } \
  NS_IMETHOD SetNewTabURL(const nsACString & aNewTabURL) override { return _to SetNewTabURL(aNewTabURL); } \
  NS_IMETHOD GetOverridden(bool *aOverridden) override { return _to GetOverridden(aOverridden); } \
  NS_IMETHOD ResetNewTabURL(void) override { return _to ResetNewTabURL(); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIABOUTNEWTABSERVICE(_to) \
  NS_IMETHOD GetNewTabURL(nsACString & aNewTabURL) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetNewTabURL(aNewTabURL); } \
  NS_IMETHOD SetNewTabURL(const nsACString & aNewTabURL) override { return !_to ? NS_ERROR_NULL_POINTER : _to->SetNewTabURL(aNewTabURL); } \
  NS_IMETHOD GetOverridden(bool *aOverridden) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetOverridden(aOverridden); } \
  NS_IMETHOD ResetNewTabURL(void) override { return !_to ? NS_ERROR_NULL_POINTER : _to->ResetNewTabURL(); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsAboutNewTabService : public nsIAboutNewTabService
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIABOUTNEWTABSERVICE

  nsAboutNewTabService();

private:
  ~nsAboutNewTabService();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsAboutNewTabService, nsIAboutNewTabService)

nsAboutNewTabService::nsAboutNewTabService()
{
  /* member initializers and constructor code */
}

nsAboutNewTabService::~nsAboutNewTabService()
{
  /* destructor code */
}

/* attribute ACString newTabURL; */
NS_IMETHODIMP nsAboutNewTabService::GetNewTabURL(nsACString & aNewTabURL)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP nsAboutNewTabService::SetNewTabURL(const nsACString & aNewTabURL)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute bool overridden; */
NS_IMETHODIMP nsAboutNewTabService::GetOverridden(bool *aOverridden)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void resetNewTabURL (); */
NS_IMETHODIMP nsAboutNewTabService::ResetNewTabURL()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIAboutNewTabService_h__ */
