/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMUIEvent.idl"

[builtinclass, uuid(05aaf2cf-27c3-4d3d-9e55-4c6a4cf700ec)]
interface nsIDOMScrollAreaEvent : nsIDOMUIEvent
{
  // Scroll area client rect
  readonly attribute float x;
  readonly attribute float y;
  readonly attribute float width;
  readonly attribute float height;

  void initScrollAreaEvent(in DOMString aEventType,
                           in boolean aCanBubble,
                           in boolean aCancelable,
                           in nsIDOMWindow aView,
                           in long aDetail,
                           in float x,
                           in float y,
                           in float width,
                           in float height);
};
